/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2013-05-02 09:01:42 +0900 #$
** $Revision: 4520 $
** $Author: ossinfra@FITECHLABS.CO.JP $
**/

#include "common.h"
#include "comms.h"
#include "log.h"
#include "md5.h"
#include "zbxserver.h"
#include "zbxself.h"

#include "jacommon.h"
#include "jatrapper.h"
#include "javalue.h"
#include "jagetagent.h"
#include "jaflow.h"
#include "jastr.h"
#include "jalog.h"
#include "jastatus.h"
#include "jahost.h"

extern unsigned char process_type;
extern char   serverid[JA_SERVERID_LEN];
extern int CONFIG_ZABBIX_VERSION;

void init_agent_result(JOBARG_AGENT_RESULT *ar)
{
	ar->host = NULL;
	ar->jobid = 0;
	ar->result = 0;
	ar->message = NULL;
	ar->jobstdout = NULL;
	ar->jobstderr = NULL;
	ar->ret = NULL;
	//ar->signal = 0; /*for reboot*/
}

void clean_agent_result(JOBARG_AGENT_RESULT *ar)
{
	zbx_free(ar->host);
	zbx_free(ar->message);
	zbx_free(ar->jobstdout);
	zbx_free(ar->jobstderr);
	zbx_free(ar->ret);
}

void init_exec_request(JOBARG_EXEC_REQUEST *er)
{
	int	i;

	er->username = NULL;
	er->password = NULL;
	er->jobnetid = NULL;
	er->starttime = NULL;

	for ( i=0 ; i < 1024 ; i++ ) {
		er->env[i] = NULL;
		er->value[i] = NULL;
	}
}

void clean_exec_request(JOBARG_EXEC_REQUEST *er, int cnt)
{
	int i;

	zbx_free(er->username);
	zbx_free(er->password);
	zbx_free(er->jobnetid);
	zbx_free(er->starttime);

	if ( cnt != 0 ) {
		for ( i=0 ; i < cnt ; i++) {
			zbx_free(er->env[i]);
			zbx_free(er->value[i]);
		}
	}
}

void init_jobnetinfo(JOBARG_JOBNET_INFO *ji)
{
	ji->jobnetid = NULL;
	ji->jobnetname = NULL;
	ji->jobnetruntype = 0;
	ji->jobnetstatus = 0;
	ji->jobstatus = 0;
	ji->scheduled_time = 0;
	ji->start_time = 0;
	ji->end_time = 0;
}

int get_jobnet_info(zbx_uint64_t registrynumber, JOBARG_EXEC_REQUEST *er, JOBARG_JOBNET_INFO *ji)
{
	DB_RESULT	result;
	DB_ROW		row;
	int		ret = SUCCEED;

	result = DBselect("select run_type, status, job_status, scheduled_time, start_time,"
				" end_time, jobnet_id, jobnet_name from ja_run_jobnet_summary_table"
				" where inner_jobnet_id = "ZBX_FS_UI64
				, registrynumber);

	row = DBfetch(result);
	if ( row == NULL ) {
		ja_log("JATRAPPER200054", 0, NULL, 0, registrynumber);
		ret = FAIL;
		return ret;
	}

	er->jobnetid = strdup(row[6]);
	ji->jobnetruntype = atoi(row[0]);
	ji->jobnetstatus = atoi(row[1]);
	ji->jobstatus = atoi(row[2]);
	ZBX_STR2UINT64(ji->scheduled_time, row[3]);
	ZBX_STR2UINT64(ji->start_time, row[4]);
	ZBX_STR2UINT64(ji->end_time, row[5]);
	ji->jobnetid = strdup(row[6]);
	ji->jobnetname = strdup(row[7]);
	
	return ret;

}

int time_passed_check(char *starttime)
{
	int		ret;
	time_t		now;
	struct tm	*tm;
	static char	buffer[13];

	now = time(NULL);
	tm = localtime(&now);

	zbx_snprintf(buffer, sizeof(buffer), "%.4d%.2d%.2d%.2d%.2d",
			tm->tm_year + 1900,
			tm->tm_mon + 1,
			tm->tm_mday,
			tm->tm_hour,
			tm->tm_min);

	ret = strcmp(starttime, buffer);
	if ( ret <=  0 ) {
		return FAIL;
	}

	return SUCCEED;
}

int host_auth(zbx_sock_t *sock, char  *host) 
{
	const char *__function_name = "host_auth";
	char		*host_value;
	char		*hostname = NULL;
	int 		ret = SUCCEED;
	zbx_uint64_t	hostid;
	int		useip;
	DB_RESULT	result;
	DB_ROW		row;
	
	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host: [%s] ",__function_name, host);

	host_value = DBdyn_escape_string(host);

	switch (CONFIG_ZABBIX_VERSION) {
		case 1:
			// for zabbix 1.8
			result = DBselect("select hostid, useip from hosts where host='%s'"
						,host_value);
			break;
		case 2:
			// for zabbix 2.0
			result = DBselect("select i.hostid, i.useip from hosts h, interface i"
						" where h.hostid=i.hostid and i.main=1 and host='%s'"
						,host_value);
			break;
		default:
			ja_log("JATRAPPER200055", 0, NULL, 0, CONFIG_ZABBIX_VERSION);
			ret = FAIL;
			return ret;
	}

	row = DBfetch(result);

	if ( row == NULL ) {
		ja_log("JATRAPPER200001", 0, NULL, 0, host_value);
		ret = FAIL;
		return ret;
	} 

	zbx_free(host_value);

	ZBX_STR2UINT64(hostid, row[0]);
	useip = atoi(row[1]);

	DBfree_result(result);

	if ( useip == USE_IPADDRESS ) {
		/*IP address*/
		switch (CONFIG_ZABBIX_VERSION) {
			case 1:
				// for zabbix 1.8
				result = DBselect("select ip from hosts where hostid="ZBX_FS_UI64
							,hostid);
				break;
			case 2:
				// for zabbix 2.0
				result = DBselect("select ip from interface where hostid="ZBX_FS_UI64" and main=1"
							,hostid);
				break;
			default:
				ja_log("JATRAPPER200055", 0, NULL, 0, CONFIG_ZABBIX_VERSION);
				ret = FAIL;
				return ret;
		}

	} else if ( useip == USE_DNS ) {
		/*DNS*/
		switch (CONFIG_ZABBIX_VERSION) {
			case 1:
				// for zabbix 1.8
				result = DBselect("select dns from hosts where hostid="ZBX_FS_UI64
							,hostid);
				break;
			case 2:
				// for zabbix 2.0
				result = DBselect("select dns from interface where hostid="ZBX_FS_UI64" and main=1"
							,hostid);
				break;
			default:
				ja_log("JATRAPPER200055", 0, NULL, 0, CONFIG_ZABBIX_VERSION);
				ret = FAIL;
				return ret;
		}

	} else {
		ja_log("JATRAPPER200002", 0, NULL, 0, useip);
		ret = FAIL;
		return ret;
	}
	
	row = DBfetch(result);

	if ( row == NULL) {
		ja_log("JATRAPPER200003", 0, NULL, 0);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	hostname = strdup(row[0]);
	DBfree_result(result);

	if ( FAIL == (zbx_tcp_check_security(sock, hostname, 0))){
		ja_log("JATRAPPER200004", 0, NULL, 0, hostname, zbx_tcp_strerror());
		ret = FAIL;
		return ret;
	} 

	zbx_free(hostname);
	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);

	return ret;
}

int job_exec_auth(JOBARG_EXEC_REQUEST er)
{
	const char *__function_name = "job_exec_auth";
	int ret = SUCCEED;
	DB_RESULT	result;
	DB_RESULT	result2;
	DB_ROW		row;
	md5_state_t	state;
	md5_byte_t	hash[MD5_DIGEST_SIZE];
	char		*jobnet_id;
	char		*user_name;
	char		*user_name2 = NULL;
	char		*user_name3;
	char		*passwd;
	char		*hash_text = NULL;
	int		valid_flag;
	int		public_flag = 0;
	int		userid;
	int		type;
	int		usrgrpid;
	int		nbytes;
	int		i;
	size_t		sz;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()",__function_name);

	/***User check****/
	jobnet_id = DBdyn_escape_string(er.jobnetid);
	user_name = DBdyn_escape_string(er.username);

	result = DBselect("select valid_flag, user_name, public_flag from ja_jobnet_control_table where jobnet_id = '%s'", jobnet_id);

	int flg = 0;
	int cnt = 0;
	while ( NULL != (row = DBfetch(result)))
	{
		flg = 1;
		valid_flag = atoi(row[0]);
		if ( valid_flag == 1) {
			cnt = 1;
			user_name2 = row[1];
			public_flag = atoi(row[2]);
		}
	}

	if ( flg == 0) {
		ja_log("JATRAPPER200046", 0, jobnet_id, 0, jobnet_id);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	if ( cnt == 0) {
		ja_log("JATRAPPER200047", 0, jobnet_id, 0, jobnet_id);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	user_name3 = DBdyn_escape_string(user_name2);

	DBfree_result(result);

	result = DBselect("select userid, type from users where alias = '%s'", user_name);

	row = DBfetch(result);
	if ( row == NULL) {
		ja_log("JATRAPPER200048", 0, jobnet_id, 0, user_name);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		zbx_free(user_name3);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	userid = atoi(row[0]);
	type = atoi(row[1]);

	if ( type == USER_TYPE_SUPER_ADMIN || public_flag == 1) {
		DBfree_result(result);
		goto passwdcheck;
	}

	DBfree_result(result);

	result = DBselect("select usrgrpid from users_groups where usrgrpid in"
				" (select usrgrpid from users_groups where userid = %d) and usrgrpid in"
				" (select usrgrpid from users_groups where userid in"
				" (select userid from users where alias = '%s'))", userid, user_name3);

	flg = 0;
	cnt = 0;
	while ( NULL != (row = DBfetch(result)))
	{
		flg = 1;
		usrgrpid = atoi(row[0]);
		result2 = DBselect("select g.usrgrpid from users_groups g, usrgrp u"
					" where g.usrgrpid=u.usrgrpid and g.usrgrpid = %d and u.users_status=0", usrgrpid);
		row = DBfetch(result2);
		if ( row == NULL) {
			cnt = cnt + 1;
		}
		DBfree_result(result2);
	}

	if ( flg == 0 ) {
		ja_log("JATRAPPER200049", 0, jobnet_id, 0, user_name);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		zbx_free(user_name3);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	if ( cnt != 0 ) {
		ja_log("JATRAPPER200050", 0, jobnet_id, 0, user_name);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		zbx_free(user_name3);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}
	
	DBfree_result(result);

passwdcheck:
	/***Password check***/
	md5_init(&state);
	passwd = DBdyn_escape_string(er.password);
	nbytes = strlen(passwd);
	md5_append(&state, (const md5_byte_t *)passwd, nbytes);
	md5_finish(&state, hash);

	sz = MD5_DIGEST_SIZE * 2 + 1;
	hash_text = zbx_malloc(hash_text, sz);

	for (i = 0; i < MD5_DIGEST_SIZE; i++)
	{
		zbx_snprintf(&hash_text[i << 1], sz - (i << 1), "%02x", hash[i]);
	}

	result = DBselect("select userid, alias from users"
				" where alias = '%s'"
				" and passwd = '%s'", user_name, hash_text);

	row = DBfetch(result);
	if ( row == NULL) {
		ja_log("JATRAPPER200008", 0, jobnet_id, 0);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		zbx_free(user_name3);
		zbx_free(passwd);
		zbx_free(hash_text);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	DBfree_result(result);
	zbx_free(jobnet_id);
	zbx_free(user_name);
	zbx_free(user_name3);
	zbx_free(passwd);
	zbx_free(hash_text);

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

int update_db_status(JOBARG_AGENT_RESULT ar)
{
	const char *__function_name = "update_db_status";
	int 		ret = SUCCEED;
	int		result;
	int		status;
	int		job_type;
	int		skipflag = 1;
	char		*range = NULL;
	char		*hostname = NULL;
	zbx_uint64_t	inner_jobnet_id;
	DB_RESULT	result_j;
	DB_ROW		row;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()",__function_name);

	if ( ar.method == JA_AGENT_METHOD_ABORT) {

		if ( ar.result == FAILURE) {
			ja_log("JATRAPPER200042", 0, NULL, 0, ar.jobid, ar.message);

		}
		return ret;
	}

	result_j = DBselect("select inner_jobnet_id, job_type, status from ja_run_job_table"
				" where inner_job_id = "ZBX_FS_UI64" for update"
				, ar.jobid);

	row = DBfetch(result_j);
	if ( row == NULL) {
		ja_log("JATRAPPER200012", 0, NULL, ar.jobid, ar.jobid);
		DBfree_result(result_j);
		ret = FAIL;
		return ret;
	}

	ZBX_STR2UINT64(inner_jobnet_id, row[0]);
	job_type = atoi(row[1]);
	status = atoi(row[2]);
	DBfree_result(result_j);

	switch (status)
	{

		case JA_JOB_STATUS_BEGIN:
			ja_log("JATRAPPER200010", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
			status = JA_JOB_STATUS_ENDERR;
			break;
		case JA_JOB_STATUS_READY:
			ja_log("JATRAPPER200010", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
			ret = FAIL;
			return ret;
		case JA_JOB_STATUS_ABORT:
			status = JA_JOB_STATUS_RUNERR;
			break;
		case JA_JOB_STATUS_RUN:
			// result error
			if ( ar.result == FAILURE ) {
				status = JA_JOB_STATUS_RUNERR;
			// signal recieved
			/*} else if (ar.signal == 1 ) {
					status = JA_JOB_STATUS_RUNERR;*/ /*for reboot*/
			// test execution
			} else if ( ar.method == JA_AGENT_METHOD_TEST ) {
				status = JA_JOB_STATUS_END;
			} else {
				// reboot result
				/*if ( job_type == JA_JOB_TYPE_REBOOT ) {
					if ( ar.result == SUCCEED )  {
						status = JA_JOB_STATUS_END;
						break;
					} else {
						status = JA_JOB_STATUS_RUNERR;
						break;
					}
				}*/ /*for reboot*/

				//return code handling
				result_j = DBselect("select stop_code from ja_run_icon_job_table"
							" where inner_job_id = "ZBX_FS_UI64
							, ar.jobid);
				row = DBfetch(result_j);
				if ( row == NULL) {
					DBfree_result(result_j);
					status = JA_JOB_STATUS_END;
				} else {
					range = strdup(row[0]);
					DBfree_result(result_j);

					if ( 0 == ja_number_match( ar.ret, range)) {
						status = JA_JOB_STATUS_END;
					} else {
						status = JA_JOB_STATUS_RUNERR;
						skipflag = 1;
						ar.message = NULL;
						ar.message = "Return code error.";
					}
					zbx_free(range);
				}
			}
			break;
		default:
			return ret;

	}

	if ( status == JA_JOB_STATUS_RUNERR ) {
		result = ja_set_runerr(ar.jobid);
		if ( result != SUCCEED ) {
			ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
			ret = FAIL;
			return ret;
		}
	} else if ( status == JA_JOB_STATUS_ENDERR ) {
		result = ja_set_enderr(ar.jobid);
		if ( result != SUCCEED ) {
			ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
			ret = FAIL;
			return ret;
		} else {
			ja_joblog(JC_JOB_ERR_END, inner_jobnet_id, ar.jobid);
		}
		return ret;
	}

	/**Aftercare**/
	if ( SUCCEED != ja_set_value_after(ar.jobid, inner_jobnet_id, MESSAGE, ar.message))
	{
		ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
		ret = FAIL;
		return ret;

	}

	if ( SUCCEED != ja_set_value_after(ar.jobid, inner_jobnet_id, STD_OUT, ar.jobstdout))
	{
		ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
		ret = FAIL;
		return ret;

	}

	if ( SUCCEED != ja_set_value_after(ar.jobid, inner_jobnet_id, STD_ERR, ar.jobstderr))
	{
		ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
		ret = FAIL;
		return ret;

	}

	if ( SUCCEED != ja_set_value_after(ar.jobid, inner_jobnet_id, RET, ar.ret))
	{
		ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
		ret = FAIL;
		return ret;

	}

	if ( status == JA_JOB_STATUS_END && job_type != JA_JOB_TYPE_ABORT ) {
		// table unlock
		if ( job_type == JA_JOB_TYPE_REBOOT ) {
			hostname = get_agent_hostname(ar.jobid, inner_jobnet_id);
			if (hostname == NULL) {
				ret = FAIL;
				return ret;
			}
			if ( SUCCEED != ja_host_unlock(hostname)) {
				ja_log("JATRAPPER200056", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
				ret = FAIL;
				return ret;
			}
 		}

		if ( SUCCEED != ja_flow(ar.jobid, JA_FLOW_TYPE_NORMAL))
		{
			ret = FAIL;
			return ret;
		}
	}

	if  ( status == JA_JOB_STATUS_RUNERR ) {
		result = ja_set_runerr(ar.jobid);
		if ( result != SUCCEED ) {
			ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
			ret = FAIL;
			return ret;
		} else {
			ja_joblog(JC_JOB_ERR_END, inner_jobnet_id, ar.jobid);
		}
		if ( skipflag == 0) {
			ja_log("JATRAPPER200043", inner_jobnet_id, NULL, ar.jobid, ar.jobid);
		}
	}

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

int register_db_table(JOBARG_EXEC_REQUEST er, zbx_uint64_t *inner_jobnet_id, int cnt)
{
	const char *__function_name = "register_db_table";
	char		*user_name = NULL;
	char		*jobnet_name = NULL;
	char		*memo = NULL;
	char		*jobnetid;
	char		*env;
	char		*value;
	int 		ret = SUCCEED;
	int		res;
	int		public_flag=0;
	int		run_type;
	int		i;
	zbx_uint64_t	next_id;
	zbx_uint64_t	update_date;
	zbx_uint64_t	scheduled_time;
	DB_RESULT	result;
	DB_ROW		row;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s() jobnetid: [%s]",__function_name, er.jobnetid);

	jobnetid = DBdyn_escape_string(er.jobnetid);
	result = DBselect("select update_date, created_date, public_flag, user_name, jobnet_name, memo" 
				" from ja_jobnet_control_table" 
				" where jobnet_id='%s'"
				" and valid_flag=1"
				, jobnetid);
	row = DBfetch(result);
	if ( row == NULL) {
		ja_log("JATRAPPER200014", 0, jobnetid, 0, jobnetid);
		zbx_free(jobnetid);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	ZBX_STR2UINT64(update_date, row[0]);
	public_flag = atoi(row[2]);
	user_name = DBdyn_escape_string(row[3]);
	jobnet_name = DBdyn_escape_string(row[4]);
	memo = DBdyn_escape_string(row[5]);
	DBfree_result(result);

	if (er.starttime != NULL) {
		run_type = JA_JOBNET_RUN_TYPE_SCHEDULED;
		ZBX_STR2UINT64(scheduled_time, er.starttime);
	} else {
		run_type = JA_JOBNET_RUN_TYPE_IMMEDIATE;
		scheduled_time = 0;
	}

	next_id = get_next_id(JA_RUN_ID_JOBNET_EX, 0, jobnetid);

	if ( next_id == 0 )
	{
		ja_log("JATRAPPER200015", 0, jobnetid, 0);
		zbx_free(jobnetid);
		zbx_free(user_name);
		zbx_free(jobnet_name);
		zbx_free(memo);
		ret = FAIL;
		return ret;
	}

	res = DBexecute("insert into ja_run_jobnet_table"
			" (inner_jobnet_id, inner_jobnet_main_id,"
			" update_date, run_type, main_flag, status, scheduled_time, public_flag,"
			" jobnet_id, user_name, jobnet_name, memo)"
			" values ( "ZBX_FS_UI64", "ZBX_FS_UI64", "ZBX_FS_UI64","
			" %d, 0, 0, "ZBX_FS_UI64", %d, '%s', '%s', '%s', '%s')"
			, next_id, next_id, update_date, run_type, scheduled_time, public_flag, jobnetid, user_name, jobnet_name, memo);
	if ( ZBX_DB_OK > res) {
		ja_log("JATRAPPER200016", 0, jobnetid, 0, jobnetid);
		zbx_free(jobnetid);
		zbx_free(user_name);
		zbx_free(jobnet_name);
		zbx_free(memo);
		ret = FAIL;
		return ret;
	}

	for ( i=0 ; i < cnt ; i++ ) {
		env = NULL;
		value = NULL;

		env = DBdyn_escape_string(er.env[i]);
		value = DBdyn_escape_string(er.value[i]);
		
		res = DBexecute("insert into ja_value_before_jobnet_table"
					" (inner_jobnet_id, value_name, before_value)"
					" values ( "ZBX_FS_UI64", '%s', '%s' )"
					, next_id, env, value);
		if ( ZBX_DB_OK > res) {
			ja_log("JATRAPPER200016", 0, jobnetid, 0, jobnetid);
			zbx_free(jobnetid);
			zbx_free(user_name);
			zbx_free(jobnet_name);
			zbx_free(memo);
			zbx_free(env);
			zbx_free(value);
			ret = FAIL;
			return ret;
		}
		zbx_free(env);
		zbx_free(value);
	}

	*inner_jobnet_id = next_id;
	zbx_free(jobnetid);
	zbx_free(user_name);
	zbx_free(jobnet_name);
	zbx_free(memo);
	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

void reply_jobresult_response(zbx_sock_t *sock, int ret, char *message)
{

	struct zbx_json 	json;

	/*JSON Data*/
	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
	zbx_json_addstring(&json, JA_PROTO_TAG_KIND, JA_PROTO_VALUE_JOBRESULT_RES, ZBX_JSON_TYPE_STRING);
	zbx_json_adduint64(&json, JA_PROTO_TAG_VERSION, JA_PROTO_VALUE_VERSION_1); /*data version*/
	zbx_json_addstring(&json, JA_PROTO_TAG_SERVERID, serverid, ZBX_JSON_TYPE_STRING);
	zbx_json_addobject(&json, JA_PROTO_TAG_DATA);
	zbx_json_adduint64(&json, JA_PROTO_TAG_RESULT, FAIL == ret ? 1 : 0);
	zbx_json_addstring(&json, JA_PROTO_TAG_MESSAGE, message, ZBX_JSON_TYPE_STRING);
	zbx_json_close(&json);
	zbx_json_close(&json);

	zabbix_log(LOG_LEVEL_DEBUG, "JSON before sending [%s]", json.buffer);

	if (FAIL == (ret = zbx_tcp_send_to(sock, json.buffer, CONFIG_TIMEOUT)))
	{
		ja_log("JATRAPPER200038", 0, NULL, 0, zbx_tcp_strerror());
		ret = NETWORK_ERROR;
	}

	zbx_json_free(&json);
}

void reply_jobnetrun_response(zbx_sock_t *sock, int ret, char *message)
{
	struct zbx_json 	json;

	/*JSON Data*/
	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
	zbx_json_addstring(&json, JA_PROTO_TAG_KIND, JA_PROTO_VALUE_JOBNETRUN_RES, ZBX_JSON_TYPE_STRING);
	zbx_json_adduint64(&json, JA_PROTO_TAG_VERSION, JA_PROTO_VALUE_VERSION_1); /*data version*/
	zbx_json_addstring(&json, JA_PROTO_TAG_SERVERID, serverid, ZBX_JSON_TYPE_STRING);
	zbx_json_addobject(&json, JA_PROTO_TAG_DATA);
	zbx_json_adduint64(&json, JA_PROTO_TAG_RESULT, FAIL == ret ? 1 : 0);
	zbx_json_addstring(&json, JA_PROTO_TAG_MESSAGE, message, ZBX_JSON_TYPE_STRING);
	zbx_json_close(&json);
	zbx_json_close(&json);

	zabbix_log(LOG_LEVEL_DEBUG, "JSON before sending [%s]", json.buffer);

	if (FAIL == (ret = zbx_tcp_send_to(sock, json.buffer, CONFIG_TIMEOUT)))
	{
		ja_log("JATRAPPER200038", 0, NULL, 0, zbx_tcp_strerror());
		ret = NETWORK_ERROR;
	}

	zbx_json_free(&json);

}

void reply_jobnetstatusrq_response(zbx_sock_t *sock, int ret, JOBARG_JOBNET_INFO *ji, char *message)
{

	struct zbx_json 	json;

	/*JSON Data*/
	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
	zbx_json_addstring(&json, JA_PROTO_TAG_KIND, JA_PROTO_VALUE_JOBNETSTATUSRQ_RES, ZBX_JSON_TYPE_STRING);
	zbx_json_adduint64(&json, JA_PROTO_TAG_VERSION, JA_PROTO_VALUE_VERSION_1); /*data version*/
	zbx_json_addstring(&json, JA_PROTO_TAG_SERVERID, serverid, ZBX_JSON_TYPE_STRING);
	zbx_json_addobject(&json, JA_PROTO_TAG_DATA);
	zbx_json_adduint64(&json, JA_PROTO_TAG_RESULT, FAIL == ret ? 1 : 0);

	if ( ret != FAIL ) {
		zbx_json_addstring(&json, JA_PROTO_TAG_JOBNETID, ji->jobnetid, ZBX_JSON_TYPE_STRING);
		zbx_json_addstring(&json, JA_PROTO_TAG_JOBNETNAME, ji->jobnetname, ZBX_JSON_TYPE_STRING);
		zbx_json_adduint64(&json, JA_PROTO_TAG_SCHEDULEDTIME, ji->scheduled_time);
		zbx_json_adduint64(&json, JA_PROTO_TAG_STARTTIME, ji->start_time);
		zbx_json_adduint64(&json, JA_PROTO_TAG_ENDTIME, ji->end_time);
		zbx_json_adduint64(&json, JA_PROTO_TAG_JOBNETRUNTYPE, ji->jobnetruntype);
		zbx_json_adduint64(&json, JA_PROTO_TAG_JOBNETSTATUS, ji->jobnetstatus);
		zbx_json_adduint64(&json, JA_PROTO_TAG_JOBSTATUS, ji->jobstatus);
	} else {
		zbx_json_addstring(&json, JA_PROTO_TAG_MESSAGE, message, ZBX_JSON_TYPE_STRING);
	}
	zbx_json_close(&json);
	zbx_json_close(&json);

	zabbix_log(LOG_LEVEL_DEBUG, "JSON before sending [%s]", json.buffer);

	if (FAIL == (ret = zbx_tcp_send_to(sock, json.buffer, CONFIG_TIMEOUT)))
	{
		ja_log("JATRAPPER200038", 0, NULL, 0, zbx_tcp_strerror());
		ret = NETWORK_ERROR;
	}

	zbx_json_free(&json);

}

char *evaluate_jobresult(zbx_sock_t *sock, struct zbx_json_parse *jp, int *ret)
{
	struct 		zbx_json_parse	jp_row;
	char		*message = NULL;
	char		value[MAX_STRING_LEN];
	const char	*p;
	int		version;
	static JOBARG_AGENT_RESULT	ar;
	zbx_uint64_t	jobid;

	init_agent_result(&ar);

	/*[version] check*/
	if (SUCCEED == zbx_json_value_by_name(jp, JA_PROTO_TAG_VERSION, value, sizeof(value)))
	{
		version = atoi(value);
		if ( version != JA_PROTO_VALUE_VERSION_1) {
			ja_log("JATRAPPER200017", 0, NULL, 0, JA_PROTO_VALUE_VERSION_1);
			*ret = FAIL;
			return zbx_dsprintf(message, "Received message error: [version] is not correct.");
		}
	} else {
		ja_log("JATRAPPER200018", 0, NULL, 0);
		*ret = FAIL;
		 return zbx_dsprintf(message, "Received message error: [version] not found");
	}
        
	/*[data] check*/
	if (NULL == (p = zbx_json_pair_by_name(jp, JA_PROTO_TAG_DATA))) {
		ja_log("JATRAPPER200021", 0, NULL, 0);
		*ret = FAIL;
		return zbx_dsprintf(message, "Received message error: [data] not found");
	} else {
		if (FAIL == (*ret = zbx_json_brackets_open(p, &jp_row))) {
			ja_log("JATRAPPER200022", 0, NULL, 0);
			*ret = FAIL;
			return zbx_dsprintf(message, "Received message error: Cannot open [data] object");
		} else {
			/*[host] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_HOST, value, sizeof(value)))
			{
				ar.host = strdup(value);

			} else {
				ja_log("JATRAPPER200023", 0, NULL, 0);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [host] not found");
			}

			/*[jobid] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBID, value, sizeof(value)))
			{
				ZBX_STR2UINT64(jobid, value);
				ar.jobid = jobid;
			} else {
				ja_log("JATRAPPER200024", 0, NULL, 0);
				zbx_free(ar.host);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [jobid] not found");
			}

			/*[method] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_METHOD, value, sizeof(value)))
			{
				ar.method = atoi(value);
				if ( ar.method < JA_AGENT_METHOD_NORMAL || ar.method > JA_AGENT_METHOD_KILL ) {
					ja_log("JATRAPPER200040", 0, NULL, ar.jobid, ar.method);
					*ret = FAIL;
					return zbx_dsprintf(message, "Received message error: method range error [%d]", ar.method);
				}
			} else {
				ja_log("JATRAPPER200041", 0, NULL, ar.jobid);
				zbx_free(ar.host);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [method] not found");
			}

			/*[result] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_RESULT, value, sizeof(value)))
			{
				ar.result = atoi(value);
				if ( 0 != ar.result && 1 != ar.result ) {
					ja_log("JATRAPPER200025", 0, NULL, ar.jobid, ar.result);
					*ret = FAIL;
					return zbx_dsprintf(message, "Received message error: result range error [%d]", ar.result);
				}
			} else {
				ja_log("JATRAPPER200026", 0, NULL, ar.jobid);
				zbx_free(ar.host);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [result] not found");
			}

			/*[message] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_MESSAGE, value, sizeof(value)))
			{
				ar.message = strdup(value);
			}

			/*[std_out] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBSTDOUT, value, sizeof(value)))
			{
				ar.jobstdout = strdup(value);
			}

			/*[std_err] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBSTDERR, value, sizeof(value)))
			{
				ar.jobstderr = strdup(value);
			}

			/*[return_code] check*/
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_RET, value, sizeof(value)))
			{
				ar.ret = strdup(value);
			}

			/*[signal] check*/
			/*if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_SIGNAL, value, sizeof(value)))
			{
				ar.signal = atoi(value);
			}*/  /*for reboot*/

			if ( SUCCEED == (host_auth(sock, ar.host)))
			{
				DBbegin();
				if ( SUCCEED == (update_db_status(ar)))
				{
					DBcommit();
					clean_agent_result(&ar);
					return zbx_dsprintf(message, "OK.");
				} else {
					DBrollback();
					clean_agent_result(&ar);
					*ret = FAIL;
					return zbx_dsprintf(message, "Update table error.");
 				}
			} else {
				clean_agent_result(&ar);
				return zbx_dsprintf(message, "[host] is not authenticated");
			}
		}
	}

}

char *evaluate_jobnetrun(zbx_sock_t *sock, struct zbx_json_parse *jp, int *ret)
{
	struct 		zbx_json_parse	jp_row;
	struct 		zbx_json_parse	jp_row2;
	char		*message = NULL;
	char		value[MAX_STRING_LEN];
	const char	*p;
	const char	*p2;
	const char	*p3 = NULL;
	int		version;
	int		res;
	int		i;
	zbx_uint64_t	inner_jobnet_id;
	static JOBARG_EXEC_REQUEST	er;

	init_exec_request(&er);

	if (SUCCEED == zbx_json_value_by_name(jp, JA_PROTO_TAG_VERSION, value, sizeof(value)))
	{
		version = atoi(value);
		if ( version != JA_PROTO_VALUE_VERSION_1) {
			ja_log("JATRAPPER200027", 0, NULL, 0, JA_PROTO_VALUE_VERSION_1);
			*ret = FAIL;
			return zbx_dsprintf(message, "Received message error: [version] is not correct.");
		}
	} else {
		ja_log("JATRAPPER200028", 0, NULL, 0);
		*ret = FAIL;
		return zbx_dsprintf(message, "Received message error: [version] not found");
	}

	if (NULL == (p = zbx_json_pair_by_name(jp, JA_PROTO_TAG_DATA))) {
		ja_log("JATRAPPER200021", 0, NULL, 0);
		*ret = FAIL;
		return zbx_dsprintf(message, "Received message error: [data] not found");
	} else {
		if (FAIL == (*ret = zbx_json_brackets_open(p, &jp_row))) {
			ja_log("JATRAPPER200022", 0, NULL, 0);
			*ret = FAIL;
			return zbx_dsprintf(message, "Received message error: Cannot open [data] object");

		} else {
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_USERNAME, value, sizeof(value)))
			{
				er.username = strdup(value);

			} else {
				ja_log("JATRAPPER200032", 0, NULL, 0);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [username] not found");
			}

			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_PASSWORD, value, sizeof(value)))
			{
				er.password = strdup(value);

			} else {
				ja_log("JATRAPPER200033", 0, NULL, 0);
				zbx_free(er.username);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [password] not found");
			}

			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBNETID, value, sizeof(value)))
			{
				er.jobnetid = strdup(value);

			} else {
				ja_log("JATRAPPER200034", 0, NULL, 0);
				zbx_free(er.username);
				zbx_free(er.password);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [jobnetid] not found");
			}

			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_STARTTIME, value, sizeof(value)))
			{
				er.starttime = strdup(value);
				res = time_passed_check(er.starttime);
				if ( res == FAIL ) {
					zbx_free(er.username);
					zbx_free(er.password);
					zbx_free(er.jobnetid);
					zbx_free(er.starttime);
					*ret = FAIL;
					return zbx_dsprintf(message, "Received message error: [start_time] already passed.");
				}
			}

			i = 0;
			if (NULL != (p2 = zbx_json_pair_by_name(&jp_row, JA_PROTO_TAG_ENV))) 
			{

				if (FAIL == (*ret = zbx_json_brackets_open(p2, &jp_row2))) {
					ja_log("JATRAPPER200051", 0, NULL, 0);
					*ret = FAIL;
					return zbx_dsprintf(message, "Received message error: Cannot open [env] object");
				} else {
					while (NULL != (p3 = zbx_json_pair_next(&jp_row2, p3, value, sizeof(value))))
					{
						er.env[i] = strdup(value);
						zbx_json_value_by_name(&jp_row2, er.env[i], value, sizeof(value));
						er.value[i] = strdup(value);
						i++;
					}
					er.env[i] = '\0';
				}
			}

			if ( SUCCEED == (job_exec_auth(er)))
			{
				DBbegin();
				if ( SUCCEED == (register_db_table(er, &inner_jobnet_id, i)))
				{
					DBcommit();
					clean_exec_request(&er, i);
					return zbx_dsprintf(message, "Registry number :  ["ZBX_FS_UI64"]" ,inner_jobnet_id);
				} else {
					DBrollback();
					clean_exec_request(&er, i);
					*ret = FAIL;
					return zbx_dsprintf(message, "Insert table error.");
 				}
							
			} else {
				clean_exec_request(&er, i);
				*ret = FAIL;
				return zbx_dsprintf(message, "Authentication failure.");
			}
		}
	}

}

char *evaluate_jobnetstatusrq(zbx_sock_t *sock, struct zbx_json_parse *jp, int *ret, JOBARG_JOBNET_INFO *ji)
{
	struct 		zbx_json_parse	jp_row;
	char		*message = NULL;
	char		value[MAX_STRING_LEN];
	const char	*p;
	int		version;
	zbx_uint64_t	registrynumber;
	static JOBARG_EXEC_REQUEST	er;

	init_exec_request(&er);

	if (SUCCEED == zbx_json_value_by_name(jp, JA_PROTO_TAG_VERSION, value, sizeof(value)))
	{
		version = atoi(value);
		if ( version != JA_PROTO_VALUE_VERSION_1) {
			ja_log("JATRAPPER200027", 0, NULL, 0, JA_PROTO_VALUE_VERSION_1);
			*ret = FAIL;
			return zbx_dsprintf(message, "Received message error: [version] is not correct.");
		}
	} else {
		ja_log("JATRAPPER200028", 0, NULL, 0);
		*ret = FAIL;
		return zbx_dsprintf(message, "Received message error: [version] not found");
	}

	if (NULL == (p = zbx_json_pair_by_name(jp, JA_PROTO_TAG_DATA))) {
		ja_log("JATRAPPER200029", 0, NULL, 0);
		*ret = FAIL;
		return zbx_dsprintf(message, "Received message error: [data] not found");
	} else {
		if (FAIL == (*ret = zbx_json_brackets_open(p, &jp_row))) {
			ja_log("JATRAPPER200030", 0, NULL, 0);
			*ret = FAIL;
			return zbx_dsprintf(message, "Received message error: Cannot open [data] object");

		} else {
			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_USERNAME, value, sizeof(value)))
			{
				er.username = strdup(value);

			} else {
				ja_log("JATRAPPER200032", 0, NULL, 0);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [username] not found");
			}

			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_PASSWORD, value, sizeof(value)))
			{
				er.password = strdup(value);

			} else {
				ja_log("JATRAPPER200033", 0, NULL, 0);
				zbx_free(er.username);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [password] not found");
			}

			if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_REGISTRYNUMBER, value, sizeof(value)))
			{
				ZBX_STR2UINT64(registrynumber, value);

			} else {
				ja_log("JATRAPPER200052", 0, NULL, 0);
				zbx_free(er.username);
				zbx_free(er.password);
				*ret = FAIL;
				return zbx_dsprintf(message, "Received message error: [registrynumber] not found");
			}

			if (SUCCEED  != get_jobnet_info(registrynumber, &er, ji))
			{
				ja_log("JATRAPPER200053", 0, NULL, 0);
				zbx_free(er.username);
				zbx_free(er.password);
				*ret = FAIL;
				return zbx_dsprintf(message, "[registrynumber] is already cleaned up in DB");

			}

			if ( SUCCEED == (job_exec_auth(er)))
			{
				zbx_free(er.username);
				zbx_free(er.password);
				*ret = SUCCEED;
				return zbx_dsprintf(message, "OK.");
							
			} else {
				zbx_free(er.username);
				zbx_free(er.password);
				*ret = FAIL;
				return zbx_dsprintf(message, "Authentication failure.");
			}
		}
	}

}

static int process_jatrap(zbx_sock_t *sock, char *s)
{
	int 		ret = SUCCEED;
	struct 		zbx_json_parse jp;
	char		value[MAX_STRING_LEN];
	char		*message = NULL;
	static 		JOBARG_JOBNET_INFO	ji;

	zbx_rtrim(s, " \r\n");
	zabbix_log(LOG_LEVEL_DEBUG, "Jatrapper got [%s] len %zd", s, strlen(s));

	/*JSON Format check*/
	if ( SUCCEED != zbx_json_open(s, &jp)) {
		ja_log("JATRAPPER200037", 0, NULL, 0);
		message = zbx_dsprintf(message, "Received message error: [JSON format error]");
		ret = FAIL;
		reply_jobresult_response(sock, ret, message);
		zbx_free(message);
		return ret;
	}

	/*[kind] check*/
	if (SUCCEED != zbx_json_value_by_name(&jp, JA_PROTO_TAG_KIND, value, sizeof(value))) {
		ja_log("JATRAPPER200036", 0, NULL, 0);
		message = zbx_dsprintf(message, "Received message error: [kind] not found");
		ret = FAIL;
		reply_jobresult_response(sock, ret, message);
		zbx_free(message);
		return ret;
	}

	if (0 == strcmp(value, JA_PROTO_VALUE_JOBRESULT)) {
		/*from Job agent*/
		message = evaluate_jobresult(sock, &jp, &ret);
		reply_jobresult_response(sock, ret, message);
	} else if (0 == strcmp(value, JA_PROTO_VALUE_JOBNETRUN)) {
		/*from external I/F*/
		message = evaluate_jobnetrun(sock, &jp, &ret);
		reply_jobnetrun_response(sock, ret, message);
	} else if (0 == strcmp(value, JA_PROTO_VALUE_JOBNETSTATUSRQ)) {
		/*from jobnet-status request*/
		init_jobnetinfo(&ji);
		message = evaluate_jobnetstatusrq(sock, &jp, &ret, &ji);
		reply_jobnetstatusrq_response(sock, ret, &ji, message);
	} else {
		ja_log("JATRAPPER200035", 0, NULL, 0);
		message = zbx_dsprintf(message, "Received message error: [kind] is not correct.");
		ret = FAIL;
		reply_jobresult_response(sock, ret, message);
	}

	zbx_free(message);
	return ret;
}

static void process_jatrapper(zbx_sock_t * sock)
{
	char *data;

    if (SUCCEED != zbx_tcp_recv_to(sock, &data, CONFIG_TIMEOUT))
        return;

    DBconnect(ZBX_DB_CONNECT_NORMAL);

    process_jatrap(sock, data);

    DBclose();
}

void main_jatrapper_loop(zbx_sock_t * s)
{
    const char *__function_name = "main_jatrapper_loop";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);

    zbx_setproctitle("%s [connecting to the database]",
                     get_process_type_string(process_type));

    for (;;) {
        zbx_setproctitle("%s [waiting for connection]",
                         ja_get_process_type_string(process_type));

        ja_update_selfmon_counter(ZBX_PROCESS_STATE_IDLE);

        if (SUCCEED == zbx_tcp_accept(s)) {
            ja_update_selfmon_counter(ZBX_PROCESS_STATE_BUSY);

            zbx_setproctitle("%s [processing data]",
                             get_process_type_string(process_type));

            process_jatrapper(s);

            zbx_tcp_unaccept(s);
        } else {
	    ja_log("JATRAPPER200039", 0, NULL, 0);
	}
    }
}
